module videoVGA (
	input clk,			// 25 MHz
	output hsyn,		// Horizontal sync
	output vsyn,		// Vertical sync
	output hde,			// Horizontal Display enable
	output vde,			// Vertical Display enable
	output [3:0]video,	// Video output (RGB)
	output [15:0]va,	// Video address
	output vrd,			// Video read
	input [15:0]d,		// input data for video read
	input [15:0]palette	// pixel palette (4-bits per entry)
);

parameter VIDEOADDR=16'h6A00; // End of SRAM

// Horizontal counters
reg [9:0]hc=0;
wire hctc=&(hc|(~799));
reg hde=0;
wire hdetc=&(hc|(~639));
reg hsyn=1;

always @(posedge clk) begin
	hc <= hctc ? 0 : hc+1;
	hde <= hctc ? 1 : (hdetc ? 0 : hde);
	hsyn= (hc==655) ? 0 : ((hc==751)? 1 : hsyn);
end

// Vertical counters
reg [9:0]vc=440;
wire vctc=hctc&(&(vc|(~524)));
reg vde=1;
wire vdetc=hctc&(&(vc|(~479)));
reg vsyn=1;

always @(posedge clk) begin
	vc <= vctc ? 0 : vc + hctc;
	vde <= vctc ? 1 : (vdetc ? 0 : vde);
	vsyn <= hctc&(vc==489) ? 0 :(hctc&(vc==491) ? 1 : vsyn);
end

// Address counter
reg [15:0]va=VIDEOADDR;
wire incaddr=(&hc[2:0]) & hde & vde;
always @(posedge clk) va <= ~vsyn ? VIDEOADDR : (va + incaddr);

// Video read
assign vrd = hde & vde & (hc[2:0]==0);

// Video shift
reg [7:0]shh=0;
reg [7:0]shl=0;
always @(posedge clk) begin
	shh <= vrd ? d[15:8] : {shh[6:0],1'b0};
	shl <= vrd ? d[ 7:0] : {shl[6:0],1'b0};
end
wire [1:0]pixel = {shh[7],shl[7]};

reg dde=0; // delayed display enable
always @(posedge clk) dde<=hde&vde;

// Palette
wire [3:0]pal[0:3];
assign pal[0]=palette[3:0]; 	// pixel 00
assign pal[1]=palette[7:4]; 	// pixel 01
assign pal[2]=palette[11:8]; 	// pixel 10
assign pal[3]=palette[115:12];	// pixel 11

reg [3:0]video;
always @(posedge clk) video<=dde ? pal[pixel]: 4'b0000;

endmodule

