//-------------------------------------------------------------------
//-------------------------------------------------------------------

`include "system.v"


module tb();

parameter FCLK=25000000;
parameter BAUD=1500000;
localparam DIVI=(FCLK*10+(BAUD/2))/BAUD; 

//-- Registros con señales de entrada
reg clk;
reg reset;
reg rxd;
wire txd;

//-- Instanciamos
wire [15:0]xd;
wire [15:0]xa;
wire xweb;
wire xoeb;

SYSTEM #(.FCLK(FCLK), .BAUD(BAUD)) sys0 ( .clk(clk), .reset(reset), .rxd(rxd), .txd(txd),
	.xd(xd), .xa(xa), .xweb(xweb), .xoeb(xoeb)

 );

// Async RAM
reg [15:0]sram[0:65535];
always @(negedge xweb) sram[xa]<=xd;
assign xd=xoeb ? 16'hZ : sram[xa];

// Reloj periódico
always #5 clk=~clk;

//-- Proceso al inicio
initial begin
	//-- Fichero donde almacenar los resultados
	$dumpfile("tb.vcd");
	$dumpvars(0, tb);
	reset = 1; clk=0; rxd=1;
	#80 reset = 0;
	
	// 0xFF
	#DIVI	rxd=0; // start
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1; // Stop

	// L (0x4C)
	#DIVI	rxd=0; // start
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop
	// 0x40
	#DIVI	rxd=0; // start
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop
	// 0x00
	#DIVI	rxd=0; // start
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop

	// 0x1
	#DIVI	rxd=0; // start
	#DIVI	rxd=1;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop
	// 0x00
	#DIVI	rxd=0; // start
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop

	// 0x40
	#DIVI	rxd=0; // start
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop
	// 0x00
	#DIVI	rxd=0; // start
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=0;
	#DIVI	rxd=1; // Stop
	
	// 0xFF
	#DIVI	rxd=0; // start
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1; // Stop
	// 0xFF
	#DIVI	rxd=0; // start
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1;
	#DIVI	rxd=1; // Stop

	//# 319 $display("FIN de la simulacion");
	# 1000000 $finish;
	//# 1000 $finish;
end



endmodule


